@@ -158,7 +158,7 @@ def extract_input_media(run_dict: Dict[str, Any]) -> Dict[str, Any]:
         "files": [],
     }
 
-    input = run_dict.get("input", [])
+    input = run_dict.get("input", {})
     input_media["images"].extend(input.get("images", []))
     input_media["videos"].extend(input.get("videos", []))
     input_media["audios"].extend(input.get("audios", []))
