@@ -12,7 +12,15 @@ except ImportError:
 
 
 class AgentQLTools(Toolkit):
-    def __init__(self, api_key: Optional[str] = None, scrape: bool = True, agentql_query: str = "", **kwargs):
+    def __init__(
+        self,
+        api_key: Optional[str] = None,
+        enable_scrape_website: bool = True,
+        enable_custom_scrape_website: bool = False,
+        all: bool = False,
+        agentql_query: str = "",
+        **kwargs,
+    ):
         self.api_key = api_key or getenv("AGENTQL_API_KEY")
         if not self.api_key:
             raise ValueError("AGENTQL_API_KEY not set. Please set the AGENTQL_API_KEY environment variable.")
@@ -20,11 +28,12 @@ class AgentQLTools(Toolkit):
         self.agentql_query = agentql_query
 
         tools: List[Any] = []
-        if scrape:
+        if all or enable_scrape_website:
             tools.append(self.scrape_website)
-        if agentql_query:
-            log_info("Custom AgentQL query provided. Registering custom scrape function.")
-            tools.append(self.custom_scrape_website)
+        if all or enable_custom_scrape_website or (agentql_query and not all and not enable_custom_scrape_website):
+            if agentql_query:
+                log_info("Custom AgentQL query provided. Registering custom scrape function.")
+                tools.append(self.custom_scrape_website)
 
         super().__init__(name="agentql_tools", tools=tools, **kwargs)
 
