@@ -6,6 +6,8 @@ from typing import List
 
 import httpx
 
+from agno.utils.log import log_info, log_warning
+
 
 class SampleDataFileExtension(str, Enum):
     DOCX = "docx"
@@ -30,7 +32,7 @@ def download_image(url: str, output_path: str) -> bool:
         # Check if the response contains image content
         content_type = response.headers.get("Content-Type")
         if not content_type or not content_type.startswith("image"):
-            print(f"URL does not point to an image. Content-Type: {content_type}")
+            log_warning(f"URL does not point to an image. Content-Type: {content_type}")
             return False
 
         path = Path(output_path)
@@ -42,14 +44,14 @@ def download_image(url: str, output_path: str) -> bool:
                 if chunk:
                     file.write(chunk)
 
-        print(f"Image successfully downloaded and saved to '{output_path}'.")
+        log_info(f"Image successfully downloaded and saved to '{output_path}'.")
         return True
 
     except httpx.HTTPError as e:
-        print(f"Error downloading the image: {e}")
+        log_warning(f"Error downloading the image: {e}")
         return False
     except IOError as e:
-        print(f"Error saving the image to '{output_path}': {e}")
+        log_warning(f"Error saving the image to '{output_path}': {e}")
         return False
 
 
@@ -109,7 +111,7 @@ def save_base64_data(base64_data: str, output_path: str) -> bool:
         with open(path, "wb") as file:
             file.write(decoded_data)
 
-        print(f"Data successfully saved to '{path}'.")
+        log_info(f"Data successfully saved to '{path}'.")
         return True
     except Exception as e:
         raise Exception(f"An unexpected error occurred while saving data to '{output_path}': {e}")
@@ -131,25 +133,25 @@ def wait_for_media_ready(url: str, timeout: int = 120, interval: int = 5, verbos
     max_attempts = timeout // interval
 
     if verbose:
-        print("Media generated! Waiting for upload to complete...")
+        log_info("Media generated! Waiting for upload to complete...")
 
     for attempt in range(max_attempts):
         try:
             response = httpx.head(url, timeout=10)
             response.raise_for_status()
             if verbose:
-                print(f"Media ready: {url}")
+                log_info(f"Media ready: {url}")
             return True
         except httpx.HTTPError:
             pass
 
         if verbose and (attempt + 1) % 3 == 0:
-            print(f"Still processing... ({(attempt + 1) * interval}s elapsed)")
+            log_info(f"Still processing... ({(attempt + 1) * interval}s elapsed)")
 
         time.sleep(interval)
 
     if verbose:
-        print(f"Timeout waiting for media. Try this URL later: {url}")
+        log_warning(f"Timeout waiting for media. Try this URL later: {url}")
     return False
 
 
